unit RosiCompGlobalCfg;

{<
  Global configuration for all RosiComponents, tools and shared forms

  Updated version compatible with Delphi2010+.
  @author(Rosinsky Software - <http://www.rosinsky.cz/delphi>)
  @created(Jan 2013)
  @lastmod(Oct 2013)
}

interface

{$INCLUDE _delphi_ver.inc}
{$INCLUDE _rosi.inc}
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Themes, DB, Grids, PropSaveMain, rHTMLLabel, rHintWindow, ShlObj;

const
  RosiCompConfig_Auto = 0;
  RosiCompConfig_On = 1;
  RosiCompConfig_Off = -1;
  rHTMLHintImageTag = '<<HINTIMG>>';

type
  TProcOnHelp = procedure(Form: TForm; HelpContext: integer) of object;

  TrDBToolCfg = record
    SQLScript_StringFieldQuotes: string;
    SQLScript_CommandSeparator: string;
    SQLScript_BooleanTrueValue: string;
    SQLScript_BooleanFalseValue: string;
    FileDialog_Options: TOpenOptions;
    FileDialog_InitDialogEvent: TNotifyEvent;
    rDlg_DefaultFont: string;
    rDlg_DefaultSize: integer;
    rDlg_AnchorClick: TAnchorClickEvent;
    rDlg_GridDrawingStyle: TGridDrawingStyle;
    rDlg_BlockStoringGridFields: boolean;
    rpictool_LastFileName: string;
  end;

  TPropSaveCfg = record
    Suffix_BlockSaveItems: string;
    FieldList_UseNewWidth: string;
    Grids_UseGlobalSetting: boolean;
  end;

  TrGroupBoxCfg = record
    DefaultCaptionColor: TColor;
    DefaultFontColor: TColor;
    DefaultCaptionHeight: integer;
    DefaultBkColor: TColor;
    UseSimpleTabColor: boolean;
  end;

  TrDBComponentsCfg = record
    JPGAsDefault: boolean;
    VisibilityFromField_AutoHide: boolean;
    VisibilityFromField_AutoShow: boolean;
  end;

  TrDBGridCfg = record
    DefaultDrawingStyle: TGridDrawingStyle;
  end;

  TrDBFieldsCfg = record
    OnSetDefaultGridColumns: TNotifyEvent;
    SetFixedCol: boolean;
  end;

  TrDBFilterCfg = record
    LikeChar: Char;
    CompoundLikeChar: Char;
    NullEqualStr: string;
    TableSepStart: Char;
    TableSepStop: Char;
    ShowDelQuotes: boolean;
    ShowCaseSensitive: boolean;
  end;

  TrDBFindCfg = record
    ShowDelQuotes: boolean;
    OnFoundEvent: TNotifyEvent;
    LastOptIndex: integer;
    LastCaseSens: boolean;
    LastFindStr: string;
  end;

  TrDBGridPropsCfg = record
    HideCellParams: boolean;
    HideTitleFilterBtn: boolean;
    HideGridStyles: boolean;
  end;

  TrDBPrintCfg = record
    PrintData_Dir: string;
  end;

  TrDBPrintDirectCfg = record
    HMarginMM: integer;
    VMarginMM: integer;
    LineSpaceRatio: double;
    PreviewMaxWidth: integer;
  end;

  TrDBSortCfg = record
    SortFieldList: boolean;
    DefaultPath: string;
  end;

  TRTFRepDlgCfg = record
    LastLangVerIndex: integer;
    ExportExcelNoFileName: boolean;
  end;

  TLoadFieldsPropsProc = procedure(DS: TDataSet) of object;

  TrDBFieldDefaultsCfg = record
    DefaultFloatDisplayFormat: string;
    DefaultFloatEditFormat: string;
    DefaultBoolDisplayFormat: string;
    DefaultDateTimeDisplayFormat: string;
    DefaultTimeDisplayFormat: string;
    LoadFieldsPropsProc: TLoadFieldsPropsProc;
    InternalSQLLog: TStrings;
    InternalLogSeparator: string;
  end;

  TRosiCompConfig = record
    Global_PropSaveFile: TPropSaveFile;
    Global_ProcOnHelp: TProcOnHelp;
    Global_HTMLHintComponent: TrHTMLHint;
    Global_HelpDirectory: string;

    PropSaveCfg: TPropSaveCfg;
    rGroupBoxCfg: TrGroupBoxCfg;
    rDBComponentsCfg: TrDBComponentsCfg;
    rDBGridCfg: TrDBGridCfg;
    rDBToolCfg: TrDBToolCfg;
    rDBFieldDefaultsCfg: TrDBFieldDefaultsCfg;

    rDBFieldsCfg: TrDBFieldsCfg;
    rDBFilterCfg: TrDBFilterCfg;
    rDBFindCfg: TrDBFindCfg;
    rDBGridPropsCfg: TrDBGridPropsCfg;
    rDBPrintCfg: TrDBPrintCfg;
    rDBPrintDirectCfg: TrDBPrintDirectCfg;
    rDBSortCfg: TrDBSortCfg;
    RTFRepDlgCfg: TRTFRepDlgCfg;
  end;

  TGridLookStyleDef = record
    StyleName: string;
    GradientStartColor: TColor;
    GradientEndColor: TColor;
    TitleFontColor: TColor;
    TextFontColor: TColor;
    TextBkColor: TColor;
    TextBkColorActiveRow: TColor;
    TextBkColorOddRow: TColor;
  end;

  TGridLookStyles = (grs_Default, grsBlueOffice, grsBlueNavy, grsBlueVista, grsSilverLight, grsSilverDark, grsSilverMonochrome, grsBlackOffice, grsOlive,
    grsYellow, grsOrange, grsGreen, grsBrown, grsKhaki);

var
  RosiCompConfig: TRosiCompConfig;
  GridLookStyleDefs: array [0 .. 13] of TGridLookStyleDef;

function GetRosiCompIniFile: string;
procedure LoadGridLookStyleList(L: TStrings);

implementation

uses rtool, rpictool, AppDirCfg;

function GetRosiCompIniFile: string;
begin
  Result:=AppDirCfg.GetSystemFolder(CSIDL_APPDATA) + 'Embarcadero';
  ForceDirectories(Result);
  Result:=Result + '\rosicomp.ini';
end;

procedure InitGridThemeColors;
var
  Theme: TGridLookStyleDef;
  idx: integer;
begin
  idx:=0;
  Theme.StyleName:='(default)';
  Theme.GradientStartColor:=clWhite;
  Theme.GradientEndColor:=$00D6D6D6;
  Theme.TitleFontColor:=clWindowText;
  Theme.TextFontColor:=clWindowText;
  Theme.TextBkColor:=clWindow; //$00F8F8F8;
  Theme.TextBkColorActiveRow:=clInfoBk;
  Theme.TextBkColorOddRow:=clNone;
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Blue Office';
  Theme.GradientStartColor:=RGB(221, 237, 253);
  Theme.GradientEndColor:=RGB(146, 192, 224);
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(232, 239, 255);
  Theme.TextBkColorActiveRow:=RGB(254, 179, 78);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Blue Nave';
  Theme.GradientStartColor:=RGB(160, 203, 229);
  Theme.GradientEndColor:=RGB(71, 141, 190);
  Theme.TextFontColor:=RGB(0, 105, 175);
  Theme.TextBkColor:=RGB(237, 243, 247);
  Theme.TextBkColorActiveRow:=RGB(190, 213, 228);
  Theme.TextBkColorOddRow:=RGB(229, 238, 244);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Blue Vista';
  Theme.GradientStartColor:=RGB(133, 175, 192);
  Theme.GradientEndColor:=RGB(41, 107, 134);
  Theme.TitleFontColor:=clWhite;
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(246, 249, 254);
  Theme.TextBkColorActiveRow:=RGB(214, 239, 252);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Silver Light';
  Theme.GradientStartColor:=RGB(243, 244, 250);
  Theme.GradientEndColor:=RGB(156, 154, 183);
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(240, 243, 248);
  Theme.TextBkColorActiveRow:=RGB(253, 225, 141);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Silver Dark';
  Theme.GradientStartColor:=RGB(220, 219, 225);
  Theme.GradientEndColor:=RGB(151, 154, 159);
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(240, 243, 248);
  Theme.TextBkColorActiveRow:=RGB(253, 225, 141);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Silver Monochrome';
  Theme.GradientStartColor:=RGB(92, 92, 92);
  Theme.GradientEndColor:=RGB(68, 68, 68);
  Theme.TitleFontColor:=clWhite;
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(247, 247, 247);
  Theme.TextBkColorActiveRow:=RGB(152, 152, 152);
  Theme.TextBkColorOddRow:=RGB(235, 235, 235);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Black Office';
  Theme.GradientStartColor:=RGB(154, 154, 154);
  Theme.GradientEndColor:=RGB(62, 64, 62);
  Theme.TitleFontColor:=clWhite;
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(240, 243, 248);
  Theme.TextBkColorActiveRow:=RGB(253, 225, 141);
  GridLookStyleDefs[idx]:=Theme;

  { Inc(idx);
    Theme:=GridLookStyleDefs[0];
    Theme.StyleName:='Black';
    Theme.GradientStartColor:=RGB(111, 111, 111);
    Theme.GradientEndColor:=RGB(44, 44, 44);
    Theme.TitleFontColor:=RGB(250, 227, 0);
    Theme.TextFontColor:=RGB(255, 246, 86);
    Theme.TextBkColor:=RGB(49, 49, 49);
    Theme.TextBkColorActiveRow:=RGB(76, 76, 76);
    GridLookStyleDefs[idx]:=Theme;
    }

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Olive';
  Theme.GradientStartColor:=RGB(251, 255, 238);
  Theme.GradientEndColor:=RGB(183, 195, 145);
  Theme.TitleFontColor:=clBlack;
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(228, 230, 199);
  Theme.TextBkColorActiveRow:=RGB(247, 201, 105);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Yellow';
  Theme.GradientStartColor:=RGB(255, 231, 77);
  Theme.GradientEndColor:=RGB(255, 209, 68);
  Theme.TitleFontColor:=RGB(141, 46, 28);
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=clWhite;
  Theme.TextBkColorActiveRow:=RGB(187, 204, 102);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Orange';
  Theme.GradientStartColor:=RGB(255, 198, 148);
  Theme.GradientEndColor:=RGB(240, 116, 18);
  Theme.TitleFontColor:=clBlack;
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=clWhite;
  Theme.TextBkColorActiveRow:=RGB(250, 158, 88);
  Theme.TextBkColorOddRow:=RGB(247, 243, 239);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Green';
  Theme.GradientStartColor:=RGB(243, 253, 188);
  Theme.GradientEndColor:=RGB(124, 180, 34);
  Theme.TitleFontColor:=clBlack;
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(236, 250, 178);
  Theme.TextBkColorActiveRow:=RGB(201, 248, 48);
  Theme.TextBkColorOddRow:=RGB(230, 249, 151);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Brown';
  Theme.GradientStartColor:=RGB(244, 212, 125);
  Theme.GradientEndColor:=RGB(185, 135, 64);
  Theme.TitleFontColor:=RGB(131, 81, 39);
  Theme.TextFontColor:=RGB(51, 51, 51);
  Theme.TextBkColor:=RGB(253, 236, 191);
  Theme.TextBkColorActiveRow:=RGB(236, 204, 119);
  Theme.TextBkColorOddRow:=RGB(230, 249, 151);
  Theme.TextBkColorOddRow:=RGB(252, 230, 170);
  GridLookStyleDefs[idx]:=Theme;

  Inc(idx);
  Theme:=GridLookStyleDefs[0];
  Theme.StyleName:='Khaki';
  Theme.GradientStartColor:=RGB(151, 136, 111);
  Theme.GradientEndColor:=RGB(114, 99, 77);
  Theme.TitleFontColor:=RGB(217, 217, 187);
  Theme.TextFontColor:=RGB(77, 68, 51);
  Theme.TextBkColor:=RGB(234, 234, 218);
  Theme.TextBkColorActiveRow:=RGB(204, 193, 163);
  GridLookStyleDefs[idx]:=Theme;
end;

procedure LoadGridLookStyleList(L: TStrings);
var
  i: integer;
begin
  L.Clear;
  for i:=0 to High(GridLookStyleDefs) do
    L.Add(GridLookStyleDefs[i].StyleName);
end;

function GetTabBackgroundColorFromBitmap: TColor;
var
  B: TBitmap;
begin
  Result:=clBtnFace;
  if GetThemeEnabled then
    begin
      B:=TBitmap.Create;
      try
        B.Width:=128;
        B.Height:=128;
        GetThemeServices.DrawElement(B.Canvas.Handle, GetThemeServices.GetElementDetails(ttBody), Rect(0, 0, B.Width - 1, B.Height - 1), nil);
        Result:=B.Canvas.Pixels[64, 64];
      finally
        B.Free;
      end;
    end;
end;

procedure InitCfgValues;
begin
  RosiCompConfig.Global_PropSaveFile:=nil;
  RosiCompConfig.Global_ProcOnHelp:=nil;
  RosiCompConfig.Global_HTMLHintComponent:=nil;
  RosiCompConfig.Global_HelpDirectory:=GetAppPath + 'help\';

  RosiCompConfig.rDBToolCfg.SQLScript_StringFieldQuotes:='''';
  RosiCompConfig.rDBToolCfg.SQLScript_CommandSeparator:='GO';
  RosiCompConfig.rDBToolCfg.SQLScript_BooleanTrueValue:='1';
  RosiCompConfig.rDBToolCfg.SQLScript_BooleanFalseValue:='0';
  RosiCompConfig.rDBToolCfg.FileDialog_Options:=[ofEnableSizing, ofNoChangeDir, ofFileMustExist];
  RosiCompConfig.rDBToolCfg.FileDialog_InitDialogEvent:=nil;
  RosiCompConfig.rDBToolCfg.rDlg_DefaultFont:='Tahoma';
  RosiCompConfig.rDBToolCfg.rDlg_DefaultSize:=8;
  RosiCompConfig.rDBToolCfg.rDlg_AnchorClick:=nil;
  RosiCompConfig.rDBToolCfg.rDlg_GridDrawingStyle:=gdsGradient;
  RosiCompConfig.rDBToolCfg.rDlg_BlockStoringGridFields:=false;
  RosiCompConfig.rDBToolCfg.rpictool_LastFileName:='';

  RosiCompConfig.rDBFieldDefaultsCfg.DefaultFloatDisplayFormat:='# ### ##0.00';
  RosiCompConfig.rDBFieldDefaultsCfg.DefaultFloatEditFormat:='0.00';
  RosiCompConfig.rDBFieldDefaultsCfg.DefaultBoolDisplayFormat:='Yes;No';
  RosiCompConfig.rDBFieldDefaultsCfg.DefaultDateTimeDisplayFormat:='';
  RosiCompConfig.rDBFieldDefaultsCfg.DefaultTimeDisplayFormat:='h:nn';
  RosiCompConfig.rDBFieldDefaultsCfg.InternalSQLLog:=nil;
  RosiCompConfig.rDBFieldDefaultsCfg.InternalLogSeparator:=';';

  RosiCompConfig.PropSaveCfg.Suffix_BlockSaveItems:='_nostore';
  RosiCompConfig.PropSaveCfg.FieldList_UseNewWidth:='';
  RosiCompConfig.PropSaveCfg.Grids_UseGlobalSetting:=false;

  RosiCompConfig.rGroupBoxCfg.DefaultCaptionColor:=clGray;
  RosiCompConfig.rGroupBoxCfg.DefaultFontColor:=clWhite;
  RosiCompConfig.rGroupBoxCfg.DefaultCaptionHeight:=14;
  RosiCompConfig.rGroupBoxCfg.DefaultBkColor:=GetTabBackgroundColorFromBitmap;
  RosiCompConfig.rGroupBoxCfg.UseSimpleTabColor:=false;

  RosiCompConfig.rDBComponentsCfg.JPGAsDefault:=rpictool.IsJPGSupported; // and not rpictool.IsPNGSupported;
  RosiCompConfig.rDBComponentsCfg.VisibilityFromField_AutoHide:=false;
  RosiCompConfig.rDBComponentsCfg.VisibilityFromField_AutoShow:=false;

  RosiCompConfig.rDBGridCfg.DefaultDrawingStyle:=gdsThemed; // gdsGradient;

  RosiCompConfig.rDBFieldsCfg.OnSetDefaultGridColumns:=nil;
  RosiCompConfig.rDBFieldsCfg.SetFixedCol:=false;

  RosiCompConfig.rDBFilterCfg.LikeChar:='*';
  RosiCompConfig.rDBFilterCfg.CompoundLikeChar:='%';
  RosiCompConfig.rDBFilterCfg.NullEqualStr:='=';
  RosiCompConfig.rDBFilterCfg.TableSepStart:='[';
  RosiCompConfig.rDBFilterCfg.TableSepStop:=']';
  RosiCompConfig.rDBFilterCfg.ShowDelQuotes:=false;
  RosiCompConfig.rDBFilterCfg.ShowCaseSensitive:=true;

  RosiCompConfig.rDBFindCfg.ShowDelQuotes:=false;
  RosiCompConfig.rDBFindCfg.OnFoundEvent:=nil;
  RosiCompConfig.rDBFindCfg.LastOptIndex:=2;
  RosiCompConfig.rDBFindCfg.LastCaseSens:=false;
  RosiCompConfig.rDBFindCfg.LastFindStr:='';

  RosiCompConfig.rDBGridPropsCfg.HideCellParams:=false;
  RosiCompConfig.rDBGridPropsCfg.HideTitleFilterBtn:=false;
  RosiCompConfig.rDBGridPropsCfg.HideGridStyles:=false;

  RosiCompConfig.rDBPrintCfg.PrintData_Dir:='';
  RosiCompConfig.rDBPrintDirectCfg.HMarginMM:=20;
  RosiCompConfig.rDBPrintDirectCfg.VMarginMM:=20;
  RosiCompConfig.rDBPrintDirectCfg.LineSpaceRatio:=0.2;
  RosiCompConfig.rDBPrintDirectCfg.PreviewMaxWidth:=1600;

  RosiCompConfig.rDBSortCfg.SortFieldList:=true;
  RosiCompConfig.rDBSortCfg.DefaultPath:=GetAppPath;

  RosiCompConfig.RTFRepDlgCfg.LastLangVerIndex:=-1;
  RosiCompConfig.RTFRepDlgCfg.ExportExcelNoFileName:=true;
end;

initialization

InitCfgValues;
InitGridThemeColors;

end.
