unit rDBRecView_DTEdit;

{<
  rDBRecView_DTEdit is design time dialog used by rDBRecView component.

  Updated version compatible with Delphi2010+.
  Full functional demo.
  @author(Rosinsky Software - <http://www.rosinsky.cz/delphi>)
  @created(May 2006)
  @lastmod(Dec 2013)
}

interface
{$INCLUDE _delphi_ver.inc}

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, DBGrids, db, ComCtrls, Math,
  {$IFDEF DXE2P} UITypes, {$ENDIF}
  ExtCtrls, CheckLst, rDBRecView, IniFiles, rImprovedComps, PropSaveMain;

type
  { design time dialog used by rDBRecView component }
  TrDBRecView_DTEditForm = class(TForm)
    pc: TPageControl;
    tsCol: TTabSheet;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    gbEmpty: TrGroupBox;
    btClose: TButton;
    gbLeft1: TrGroupBox;
    Label1: TLabel;
    lbFields: TListBox;
    btMove: TButton;
    btMoveAll: TButton;
    Label4: TLabel;
    eSep: TEdit;
    chWOCaption: TCheckBox;
    gbRight1: TrGroupBox;
    Label2: TLabel;
    Label5: TLabel;
    btUp: TButton;
    btDown: TButton;
    btDel: TButton;
    btDelAll: TButton;
    btDelNoField: TButton;
    lbCol: TListBox;
    eTitle: TEdit;
    gbRight2: TrGroupBox;
    Label3: TLabel;
    lbExample: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    cbColor: TColorBox;
    btFont: TButton;
    chStandardFont: TCheckBox;
    gbFontStyle: TGroupBox;
    chNormal: TCheckBox;
    chBold: TCheckBox;
    chItalic: TCheckBox;
    chUnderline: TCheckBox;
    eFontSize: TrEditNum;
    gbLeft2: TrGroupBox;
    Label11: TLabel;
    lbSpec: TCheckListBox;
    gbRight3: TrGroupBox;
    Label12: TLabel;
    cbLinkType: TComboBox;
    Label19: TLabel;
    gbLeft3: TrGroupBox;
    Label6: TLabel;
    lbLink: TCheckListBox;
    gbLeft4: TrGroupBox;
    Label17: TLabel;
    cbPickLists: TComboBox;
    Label7: TLabel;
    ePickList: TMemo;
    gbRight4: TrGroupBox;
    btPickSave: TButton;
    btPickDel: TButton;
    PSFile: TPropSaveFile;
    PSMain: TPropSaveMain;
    procedure FormShow(Sender: TObject);
    procedure lbColDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure btUpClick(Sender: TObject);
    procedure btDownClick(Sender: TObject);
    procedure lbColKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure btDelClick(Sender: TObject);
    procedure btDelAllClick(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btMoveClick(Sender: TObject);
    procedure btMoveAllClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btDelNoFieldClick(Sender: TObject);
    procedure lbSpecDblClick(Sender: TObject);
    procedure lbSpecClickCheck(Sender: TObject);
    procedure lbSpecClick(Sender: TObject);
    procedure chBoldExit(Sender: TObject);
    procedure btFontClick(Sender: TObject);
    procedure lbColClick(Sender: TObject);
    procedure eTitleExit(Sender: TObject);
    procedure lbLinkDblClick(Sender: TObject);
    procedure lbLinkClickCheck(Sender: TObject);
    procedure lbLinkClick(Sender: TObject);
    procedure cbLinkTypeExit(Sender: TObject);
    procedure cbPickListsDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure cbPickListsChange(Sender: TObject);
    procedure btPickSaveClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    FLabels: TStrings;
    FFields: TStrings;
    FSpecFields: TStrings;
    FSpecFont: TFont;
    FAsLink: TStrings;
    FPickLists: TStrings;
    FPickListIni: TMemIniFile;

    procedure LoadProps;
    procedure SetProps;
    procedure AddCol(FldName, FldLabel: string);
    procedure MoveI(List: TListBox; old, new: Integer);
  public
    { Public declarations }
  end;

function ShowrDBRecViewEditor(Component: TComponent): Boolean;

implementation

{$R *.dfm}
uses TypInfo, RosiCompGlobalCfg, rtool, rdlg, rdbtool;

function ShowrDBRecViewEditor(Component: TComponent): Boolean;
var
  F: TrDBRecView_DTEditForm;
  DS: TDataSource;
  Gr: TDBGrid;
  GD: TObject;
  TabVis, i, act: Integer;
begin
  Result:=false;
  TabVis:=GetOrdProp(Component, 'GridDlgVisibleTabs');
  if not(csDesigning in Component.ComponentState) and (TabVis = 0) then
    Exit;
  Result:=true;
  F:=TrDBRecView_DTEditForm.Create(Application);
  try
    F.Caption:=Component.Name;
    F.FFields:=GetObjectProp(Component, 'Fields') as TStrings;
    F.FSpecFields:=GetObjectProp(Component, 'SpecFields') as TStrings;
    F.FSpecFont:=GetObjectProp(Component, 'SpecFont') as TFont;
    F.FAsLink:=GetObjectProp(Component, 'FieldsAsLink') as TStrings;
    F.lbFields.Items.Clear;
    F.FPickLists:=GetObjectProp(Component, 'PickLists') as TStrings;

    if not(csDesigning in Component.ComponentState) then
    begin
      // set tab visibility
      act:=0;
      for i:=F.pc.PageCount - 1 downto 0 do
      begin
        F.pc.Pages[i].TabVisible:=(Round(IntPower(2, i)) and TabVis) > 0;
        if F.pc.Pages[i].TabVisible then
          act:=i;
      end;
      F.pc.ActivePageIndex:=act;
    end;

    try
      Gr:=GetObjectProp(Component, 'DBGrid') as TDBGrid;
      DS:=GetObjectProp(Component, 'DataSource') as TDataSource;
      if Assigned(Gr) then
        GD:=Gr
      else
        if Assigned(DS) then
          GD:=DS.DataSet
        else
          GD:=nil;
      GetFldList(GD, F.lbFields.Items, [fdAllField]);
      GetFldList(GD, F.FLabels, [fdGetLabels, fdAllField]);
      F.lbSpec.Items.Assign(F.lbFields.Items);
      if F.lbSpec.Items.Count > 0 then
        F.lbSpec.ItemIndex:=0;
    except
    end;
    F.ShowModal;
  finally
    F.Free;
  end; (Component as TControl)
  .Invalidate;
  if Component is TrDBRecView then (Component as TrDBRecView)
    .SetRows;
end;

procedure TrDBRecView_DTEditForm.FormCreate(Sender: TObject);
begin
  FLabels:=TStringList.Create;
  FPickListIni:=TMemIniFile.Create('');
  PSFile.LoadFile(GetRosiCompIniFile);
end;

procedure TrDBRecView_DTEditForm.FormDestroy(Sender: TObject);
begin
  FLabels.Free;
  FPickListIni.Free;
end;

procedure TrDBRecView_DTEditForm.FormResize(Sender: TObject);
begin
  gbLeft1.Width:=pc.ClientWidth div 2;
  gbLeft2.Width:=pc.ClientWidth div 2;
  gbLeft3.Width:=pc.ClientWidth div 2;
  gbLeft4.Width:=pc.ClientWidth div 2;
end;

procedure TrDBRecView_DTEditForm.FormShow(Sender: TObject);
begin
  // pc.ActivePageIndex:=0;
  LoadProps;
end;

procedure TrDBRecView_DTEditForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SetProps;
end;

procedure TrDBRecView_DTEditForm.btCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TrDBRecView_DTEditForm.LoadProps;
var
  i, j: Integer;
  s: string;
begin
  // load Fields and SpecFields
  lbCol.Items.Clear;
  for i:=0 to FFields.Count - 1 do
  begin
    s:=FFields.Names[i];
    if s = '' then
      s:=FFields[i];
    lbCol.Items.Add(s);
  end;
  for i:=0 to lbSpec.Items.Count - 1 do
    lbSpec.Checked[i]:=false;
  for i:=0 to FSpecFields.Count - 1 do
  begin
    j:=lbSpec.Items.IndexOf(FSpecFields.Names[i]);
    if j >= 0 then
      lbSpec.Checked[j]:=true;
  end;
  btFontClick(nil);
  // load FieldsAsLink
  lbLink.Items.Assign(lbFields.Items);
  for i:=0 to FAsLink.Count - 1 do
  begin
    // check and convert old format (without values)
    s:=Trim(FAsLink.Strings[i]);
    if (Length(s) > 0) and (Pos('=', s) = 0) then
      FAsLink.Strings[i]:=s + '=' + cbLinkType.Items[0];
    // set lbLink
    j:=lbLink.Items.IndexOf(FAsLink.Names[i]);
    if j >= 0 then
      lbLink.Checked[j]:=true;
  end;
  cbPickLists.Items.Assign(lbFields.Items);
  FPickListIni.SetStrings(FPickLists);
end;

procedure TrDBRecView_DTEditForm.SetProps;
begin
  // set Fields
end;

// FIELDS proc
procedure TrDBRecView_DTEditForm.AddCol(FldName, FldLabel: string);
var
  oldItemIndex: Integer;
begin
  oldItemIndex:=lbCol.ItemIndex;
  if (oldItemIndex < 0) or (oldItemIndex > lbCol.Count - 1) then
    oldItemIndex:=lbCol.Count - 1
  else
    Inc(oldItemIndex);
  if oldItemIndex < 0 then
    oldItemIndex:=0;
  if self.chWOCaption.Checked then
    FFields.Insert(oldItemIndex, FldName)
  else
    FFields.Insert(oldItemIndex, FldName + '=' + ReplaceStr(eSep.Text, ' ', FldLabel, true, false));
  lbCol.Items.Insert(oldItemIndex, FldName);
  lbCol.ItemIndex:=oldItemIndex;
  lbColClick(nil);
end;

procedure TrDBRecView_DTEditForm.MoveI(List: TListBox; old, new: Integer);
begin
  if not MoveListBox(List, old, new) then
    Exit;
  FFields.Move(old, new);
  lbColClick(nil);
end;

procedure TrDBRecView_DTEditForm.lbColDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept:=true;
  MoveI(lbCol, lbCol.ItemIndex, Y div lbCol.ItemHeight + lbCol.TopIndex);
end;

procedure TrDBRecView_DTEditForm.btUpClick(Sender: TObject);
begin
  MoveI(lbCol, lbCol.ItemIndex, lbCol.ItemIndex - 1);
end;

procedure TrDBRecView_DTEditForm.btDownClick(Sender: TObject);
begin
  MoveI(lbCol, lbCol.ItemIndex, lbCol.ItemIndex + 1);
end;

procedure TrDBRecView_DTEditForm.lbColKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_UP) and (ssCtrl in Shift) then
  begin
    MoveI(lbCol, lbCol.ItemIndex, lbCol.ItemIndex - 1);
    Key:=0;
  end;
  if (Key = VK_DOWN) and (ssCtrl in Shift) then
  begin
    MoveI(lbCol, lbCol.ItemIndex, lbCol.ItemIndex + 1);
    Key:=0;
  end;
  if (Key = VK_DELETE) then
  begin
    btDel.Click;
    Key:=0;
  end;
end;

procedure TrDBRecView_DTEditForm.btDelClick(Sender: TObject);
var
  i: Integer;
begin
  i:=lbCol.ItemIndex;
  if i = -1 then
    Exit;
  lbCol.Items.Delete(i);
  FFields.Delete(i);
  if i > lbCol.Items.Count - 1 then
    i:=lbCol.Items.Count - 1;
  lbCol.ItemIndex:=i;
  lbColClick(nil);
end;

procedure TrDBRecView_DTEditForm.btDelAllClick(Sender: TObject);
begin
  if DlgQ('Delete all fields?') <> mrYes then
    Exit;
  lbCol.Clear;
  FFields.Clear;
  lbColClick(nil);
end;

procedure TrDBRecView_DTEditForm.btDelNoFieldClick(Sender: TObject);
var
  i: Integer;
begin
  if DlgQ('Delete all orphan fields (not existing field)?') <> mrYes then
    Exit;
  for i:=lbCol.Items.Count - 1 downto 0 do
    if lbFields.Items.IndexOf(lbCol.Items[i]) < 0 then
    begin
      lbCol.Items.Delete(i);
      FFields.Delete(i);
    end;
  lbColClick(nil);
end;

procedure TrDBRecView_DTEditForm.btMoveClick(Sender: TObject);
var
  i: Integer;
begin
  for i:=0 to lbFields.Items.Count - 1 do
    if lbFields.Selected[i] then
      AddCol(lbFields.Items[i], FLabels[i]);
end;

procedure TrDBRecView_DTEditForm.btMoveAllClick(Sender: TObject);
var
  i: Integer;
begin
  if DlgQ('Add all columns?') <> mrYes then
    Exit;
  for i:=0 to lbFields.Items.Count - 1 do
    lbFields.Selected[i]:=true;
  btMove.Click;
end;

procedure TrDBRecView_DTEditForm.lbColClick(Sender: TObject);
begin
  eTitle.Text:='';
  eTitle.Enabled:=(lbCol.ItemIndex >= 0) and not self.chWOCaption.Checked;
  if eTitle.Enabled then
    eTitle.Text:=FFields.Values[FFields.Names[lbCol.ItemIndex]];
end;

procedure TrDBRecView_DTEditForm.eTitleExit(Sender: TObject);
begin
  if lbCol.ItemIndex >= 0 then
    FFields.Values[FFields.Names[lbCol.ItemIndex]]:=eTitle.Text;
end;

procedure TrDBRecView_DTEditForm.lbSpecDblClick(Sender: TObject);
begin
  lbSpec.Checked[lbSpec.ItemIndex]:=not lbSpec.Checked[lbSpec.ItemIndex];
  lbSpecClickCheck(self);
  lbSpecClick(self);
end;

procedure TrDBRecView_DTEditForm.lbSpecClickCheck(Sender: TObject);
var
  i: Integer;
begin
  if not lbSpec.Checked[lbSpec.ItemIndex] then
  begin
    i:=FSpecFields.IndexOfName(lbSpec.Items[lbSpec.ItemIndex]);
    if i >= 0 then
      FSpecFields.Delete(i);
  end
  else
  begin
    FSpecFields.Add(lbSpec.Items[lbSpec.ItemIndex] + '=0');
  end;
end;

procedure TrDBRecView_DTEditForm.lbSpecClick(Sender: TObject);
var
  Format, s: string;
  i, fs: Integer;
  C: LongInt;
begin
  chStandardFont.Checked:=false;
  chNormal.Checked:=false;
  chBold.Checked:=false;
  chItalic.Checked:=false;
  chUnderline.Checked:=false;
  cbColor.Selected:=clNone;
  eFontSize.AsInteger:=0;
  if lbSpec.Checked[lbSpec.ItemIndex] then
  begin
    gbRight2.Enabled:=true;
    Format:=FSpecFields.Values[lbSpec.Items[lbSpec.ItemIndex]];
    i:=1;
    repeat
      s:=AnsiUpperCase(Trim(FindStrPart(Format, ';', i, false)));
      if s = '' then
        Break;
      if s = 'SF' then
        chStandardFont.Checked:=true
      else
        if s = 'N' then
          chNormal.Checked:=true
        else
          if s = 'B' then
            chBold.Checked:=true
          else
            if s = 'I' then
              chItalic.Checked:=true
            else
              if s = 'U' then
                chUnderline.Checked:=true
              else
                if IdentToColor('cl' + s, C) then
                  cbColor.Selected:=C
                else
                begin
                  fs:=StrToIntDef(s, 0);
                  if fs > 0 then
                    eFontSize.AsInteger:=fs;
                end;
      Inc(i);
    until false;
  end
  else
  begin
    gbRight2.Enabled:=false;
  end;
end;

procedure TrDBRecView_DTEditForm.chBoldExit(Sender: TObject);
var
  i: Integer;
  s, scol: string;
begin
  i:=FSpecFields.IndexOfName(lbSpec.Items[lbSpec.ItemIndex]);
  if i >= 0 then
  begin
    s:='';
    if chStandardFont.Checked then
      s:=s + 'SF;';
    if chNormal.Checked then
      s:=s + 'N;';
    if chBold.Checked then
      s:=s + 'B;';
    if chItalic.Checked then
      s:=s + 'I;';
    if chUnderline.Checked then
      s:=s + 'U;';
    if cbColor.Selected <> clNone then
    begin
      scol:='';
      if ColorToIdent(cbColor.Selected, scol) then
        s:=s + copy(scol, 3, 255);
    end;
    if eFontSize.AsInteger > 0 then
      s:=s + IntToStr(eFontSize.AsInteger) + ';';
    if s = '' then
      s:='0';
    FSpecFields.Values[lbSpec.Items[lbSpec.ItemIndex]]:=s;
  end;
end;

procedure TrDBRecView_DTEditForm.btFontClick(Sender: TObject);
var
  D: TFontDialog;
begin
  if Sender <> nil then
  begin
    D:=TFontDialog.Create(self);
    try
      D.Font.Assign(FSpecFont);
      if D.Execute then
        FSpecFont.Assign(D.Font);
    finally
      D.Free;
    end;
  end;
  lbExample.Font.Assign(FSpecFont);
end;

// LINK proc

procedure TrDBRecView_DTEditForm.lbLinkDblClick(Sender: TObject);
begin
  lbLink.Checked[lbLink.ItemIndex]:=not lbLink.Checked[lbLink.ItemIndex];
  lbLinkClickCheck(self);
  lbLinkClick(self);
end;

procedure TrDBRecView_DTEditForm.lbLinkClickCheck(Sender: TObject);
var
  i: Integer;
begin
  if not lbLink.Checked[lbLink.ItemIndex] then
  begin
    i:=FAsLink.IndexOfName(lbLink.Items[lbLink.ItemIndex]);
    if i >= 0 then
      FAsLink.Delete(i);
  end
  else
  begin
    FAsLink.Add(lbLink.Items[lbLink.ItemIndex] + '=' + cbLinkType.Items[0]);
  end;
end;

procedure TrDBRecView_DTEditForm.lbLinkClick(Sender: TObject);
begin
  cbLinkType.ItemIndex:=-1;
  gbRight3.Enabled:=lbLink.Checked[lbLink.ItemIndex];
  if gbRight3.Enabled then
    cbLinkType.ItemIndex:=cbLinkType.Items.IndexOf(FAsLink.Values[lbLink.Items[lbLink.ItemIndex]]);
end;

procedure TrDBRecView_DTEditForm.cbLinkTypeExit(Sender: TObject);
var
  i: Integer;
begin
  i:=FAsLink.IndexOfName(lbLink.Items[lbLink.ItemIndex]);
  if i >= 0 then
    FAsLink.Values[lbLink.Items[lbLink.ItemIndex]]:=cbLinkType.Text;
end;

procedure TrDBRecView_DTEditForm.cbPickListsDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  with Control as TComboBox do
  begin
    Canvas.FillRect(Rect);
    if FPickListIni.SectionExists(Items[Index]) then
      Canvas.Font.Style:=Canvas.Font.Style + [fsBold];
    Canvas.TextOut(Rect.Left + 1, Rect.Top + 1, Items[Index]);
  end;
end;

procedure TrDBRecView_DTEditForm.cbPickListsChange(Sender: TObject);
begin
  ePickList.ReadOnly:=false;
  ePickList.Lines.Clear;
  FPickListIni.ReadSectionValues(cbPickLists.Text, ePickList.Lines);
end;

procedure TrDBRecView_DTEditForm.btPickSaveClick(Sender: TObject);
var
  i: Integer;
begin
  if Sender = btPickDel then
    ePickList.Lines.Clear;
  FPickListIni.EraseSection(cbPickLists.Text);
  FPickLists.Clear;
  FPickListIni.GetStrings(FPickLists);
  if Trim(ePickList.Text) <> '' then
  begin
    FPickLists.Add('[' + cbPickLists.Text + ']');
    for i:=0 to ePickList.Lines.Count - 1 do
      FPickLists.Add(ePickList.Lines[i]);
  end;
  FPickListIni.Clear;
  FPickListIni.SetStrings(FPickLists);
  cbPickLists.Invalidate;
end;

end.
